/** \addtogroup group_ds1w Dallas 1-wire (работа с шиной Dallas 1-wire)
    \note Before use define functions or macros (перед включением модуля нужно определить следующие функции или макросы) :
     <br /><i><b>U8 ds1w_dq_rd(void)</b></i> - read DQ level (to any bit of result)
     <br /><i><b>void ds1w_dq_wr(U8 dq_level)</b></i> - write DQ level (from any bit of dq_level)
     <br /><i><b>void ds1w_delay_bit_before(void)</b></i> - delay before bit read/write
     <br /><i><b>void ds1w_delay_bit_start(void)</b></i> - delay before setting DQ level (<15usec)
     <br /><i><b>void ds1w_delay_bit_rd(void)</b></i> - delay before bit level read (~40usec)
     <br /><i><b>void ds1w_delay_bit_after(void)</b></i> - delay after bit read/write (>60usec)
     <br /><i><b>void ds1w_delay_byte_before(void)</b></i> - delay before byte read/write
     <br /><i><b>void ds1w_delay_byte_after(void)</b></i> - delay before byte read/write
     <br /><i><b>void ds1w_delay_reset_before(void)</b></i> - delay before reset pulse
     <br /><i><b>void ds1w_delay_reset(void)</b></i> - delay till reset pulse
     <br /><i><b>void ds1w_delay_presence_before(void)</b></i> - delay after reset pulse before reading presence pulse (~80usec)
     <br /><i><b>void ds1w_delay_presence_after(void)</b></i> - delay after reset pulse after reading presence pulse (~200usec)
     <br />
     <br />for AVR define <i><b>DS1W_DIR_PORT</b></i>, <i><b>DS1W_OUT_PORT</b></i>, <i><b>DS1W_IN_PORT</b></i>, <i><b>DS1W_BIT</b></i>, <i><b>DS1W_DELAY_RESET_TICKS</b></i>
  */

#if ((!defined(UL_1WIRE_INCLUDED)) || (defined __DOXYGEN__))
    #define UL_1WIRE_INCLUDED 1 /**< \brief module inclusion flag<br />флаг включения модуля \ingroup group_ds1w */

#include "ul_misc.h" /*include misc definitions/functions types*/

#define ds1w_res_ok       0 /**< \brief 1-wire reset OK (ACK received)<br />сброс 1-wire шины прошел нормально (получен ACK) \ingroup group_ds1w */
#define ds1w_res_err_sh0  1 /**< \brief 1-wire shorted to GND<br />1-wire шина замкнута на землю \ingroup group_ds1w */
#define ds1w_res_err_sh1  2 /**< \brief 1-wire shorted to VCC<br />1-wire шина замкнута на питание \ingroup group_ds1w */
#define ds1w_res_err_nack 3 /**< \brief no ACK received<br />не получен ACK \ingroup group_ds1w */

extern U8 ds1w_res_status;  /**< \brief previous Reset status<br />предыдущий результат сброса \ingroup group_ds1w */


#if (defined(USE_GCCAVR) && defined(DS1W_OUT_PORT))
    #define ds1w_dq_rd() ( DS1W_IN_PORT & DS1W_BIT )
    #define ds1w_dq_wr(dq_level) { if (dq_level) { DS1W_OUT_PORT |= DS1W_BIT; DS1W_DIR_PORT &= ~DS1W_BIT; } else { DS1W_OUT_PORT &= ~DS1W_BIT; DS1W_DIR_PORT |= DS1W_BIT; } }
    #define ds1w_delay_bit_before()      delay_tmr_ticks(1)
    #define ds1w_delay_bit_start()       delay_us(4)
    #define ds1w_delay_bit_rd()          delay_us(10)
    #define ds1w_delay_bit_after()       delay_us(70)
    #define ds1w_delay_byte_before()
    #define ds1w_delay_byte_after()
    #define ds1w_delay_reset_before()
    #define ds1w_delay_reset()           delay_tmr_ticks(DS1W_DELAY_RESET_TICKS)
    #define ds1w_delay_presence_before() delay_us(80)
    #define ds1w_delay_presence_after()  {delay_us(50); delay_us(50); delay_us(50); delay_us(50);}
#endif


/** \brief read bit to 7th bit of \b data_in<br />прочитать бит в 7й бит \b data_in \ingroup group_ds1w */
U8 ds1w_bit_rd(U8 data_in);

/** \brief write bit 0 of \b data_out<br />записать бит 0 из \b data_out \ingroup group_ds1w */
void ds1w_bit_wr(U8 data_out);

/** \brief read byte<br />прочитать байт \ingroup group_ds1w */
U8 ds1w_byte_rd(void);

/** \brief write byte<br />записать байт \ingroup group_ds1w */
void ds1w_byte_wr(U8 byte_in);

/** \brief reset 1-wire bus,return \b 0 if \b ACK received,\b 1 if \b DQ shorted to ground,\b 2 if \b DQ shorted to VCC,\b 3 if no \b ACK received or \b DQ floating
     <br />сбросить шину,возвратить \b 0 если \b ACK принят, \b 1 если \b DQ замкнут на землю, \b 2 если \b DQ замкнут на питание, \b 3 если \b ACK не принят (или обрыв \b DQ) \ingroup group_ds1w
  */
U8 ds1w_res(void);

/** \brief convert 12-bit temperature data of ds18b20 to celsius signed fixed-point (with tenth)
     <br />преобразовать 12-битное значение температуры в десятичное с фиксированной точкой (1 цифра после точки) \ingroup group_ds1w */
S16 ds1w_12bit2tenth(U16 value);

#endif
